@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -47,14 +47,14 @@ pub use self::image::*;
 
 /// CachedGraphicsData allows the graphics backend to store an arbitrary piece of data associated with
 /// an item, which is typically computed by accessing properties. The dependency_tracker is used to allow
-/// for a lazy computation. Typically backends store either compute intensive data or handles that refer to
+/// for a lazy computation. Typically back ends store either compute intensive data or handles that refer to
 /// data that's stored in GPU memory.
 pub struct CachedGraphicsData<T> {
     /// The backend specific data.
     pub data: T,
     /// The property tracker that should be used to evaluate whether the primitive needs to be re-created
     /// or not.
-    pub dependency_tracker: core::pin::Pin<Box<crate::properties::PropertyTracker>>,
+    pub dependency_tracker: Option<core::pin::Pin<Box<crate::properties::PropertyTracker>>>,
 }
 
 impl<T> CachedGraphicsData<T> {
@@ -63,12 +63,12 @@ impl<T> CachedGraphicsData<T> {
     pub fn new(update_fn: impl FnOnce() -> T) -> Self {
         let dependency_tracker = Box::pin(crate::properties::PropertyTracker::default());
         let data = dependency_tracker.as_ref().evaluate(update_fn);
-        Self { data, dependency_tracker }
+        Self { data, dependency_tracker: Some(dependency_tracker) }
     }
 }
 
-/// The RenderingCache, in combination with CachedGraphicsData, allows backends to store data that's either
-/// intensive to compute or has bad CPU locality. Backends typically keep a RenderingCache instance and use
+/// The RenderingCache, in combination with CachedGraphicsData, allows back ends to store data that's either
+/// intensive to compute or has bad CPU locality. Back ends typically keep a RenderingCache instance and use
 /// the item's cached_rendering_data() integer as index in the vec_arena::Arena.
 pub struct RenderingCache<T> {
     slab: slab::Slab<CachedGraphicsData<T>>,
@@ -93,6 +93,11 @@ impl<T> RenderingCache<T> {
         self.slab.get_mut(index)
     }
 
+    /// Returns true if a cache entry exists for the given index.
+    pub fn contains(&self, index: usize) -> bool {
+        self.slab.contains(index)
+    }
+
     /// Inserts data into the cache and returns the index for retrieval later.
     pub fn insert(&mut self, data: CachedGraphicsData<T>) -> usize {
         self.slab.insert(data)
@@ -138,28 +143,13 @@ impl FontRequest {
     pub fn merge(self, other: &FontRequest) -> Self {
         Self {
             family: self.family.or_else(|| other.family.clone()),
-            weight: self.weight.or_else(|| other.weight.clone()),
-            pixel_size: self.pixel_size.or_else(|| other.pixel_size.clone()),
-            letter_spacing: self.letter_spacing.or_else(|| other.letter_spacing.clone()),
+            weight: self.weight.or(other.weight),
+            pixel_size: self.pixel_size.or(other.pixel_size),
+            letter_spacing: self.letter_spacing.or(other.letter_spacing),
         }
     }
 }
 
-/// The FontMetrics trait is constructed from a FontRequest by the graphics backend and supplied to text related
-/// items in order to measure text.
-pub trait FontMetrics {
-    /// Returns the size of the given string in logical pixels.
-    /// When set, `max_width` means that one need to wrap the text so it does not go further than that
-    fn text_size(&self, text: &str, max_width: Option<f32>) -> Size;
-    /// Returns the height of a line of text.
-    fn line_height(&self) -> f32;
-    /// Returns the (UTF-8) byte offset in the given text that refers to the character that contributed to
-    /// the glyph cluster that's visually nearest to the given x coordinate. This is used for hit-testing,
-    /// for example when receiving a mouse click into a text field. Then this function returns the "cursor"
-    /// position.
-    fn text_offset_for_x_position<'a>(&self, text: &'a str, x: f32) -> usize;
-}
-
 #[cfg(feature = "ffi")]
 pub(crate) mod ffi {
     #![allow(unsafe_code)]
