@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -65,7 +65,7 @@ fn alloc_with_capacity<T>(capacity: usize) -> NonNull<SharedVectorInner<T>> {
 /// Return a new capacity suitable for this vector
 /// Loosely based on alloc::raw_vec::RawVec::grow_amortized.
 fn capacity_for_grow(current_cap: usize, required_cap: usize, elem_size: usize) -> usize {
-    if current_cap >= elem_size {
+    if current_cap >= required_cap {
         return current_cap;
     }
     let cap = core::cmp::max(current_cap * 2, required_cap);
@@ -118,7 +118,7 @@ impl<T> Clone for SharedVector<T> {
             {
                 self.inner.as_ref().header.refcount.fetch_add(1, atomic::Ordering::SeqCst);
             }
-            return SharedVector { inner: self.inner };
+            SharedVector { inner: self.inner }
         }
     }
 }
@@ -138,9 +138,14 @@ impl<T> SharedVector<T> {
         unsafe { self.inner.cast::<SharedVectorHeader>().as_ref().size }
     }
 
+    /// Return true if the SharedVector is empty
+    pub fn is_empty(&self) -> bool {
+        self.len() == 0
+    }
+
     /// Return a slice to the array
     pub fn as_slice(&self) -> &[T] {
-        if self.len() == 0 {
+        if self.is_empty() {
             &[]
         } else {
             // Safety: When len > 0, we know that the pointer holds an array of the size of len
@@ -171,8 +176,7 @@ impl<T: Clone> SharedVector<T> {
         let mut new_array = SharedVector::with_capacity(new_capacity);
         core::mem::swap(&mut self.inner, &mut new_array.inner);
         let mut size = 0;
-        let mut iter = new_array.into_iter();
-        while let Some(x) = iter.next() {
+        for x in new_array.into_iter() {
             assert_ne!(size, new_capacity);
             unsafe {
                 core::ptr::write(self.inner.as_mut().data.as_mut_ptr().add(size), x);
@@ -186,7 +190,13 @@ impl<T: Clone> SharedVector<T> {
     }
 
     /// Return a mutable slice to the array. If the array was shared, this will make a copy of the array.
+    #[deprecated(note = "Use make_mut_slice() instead")]
     pub fn as_slice_mut(&mut self) -> &mut [T] {
+        self.make_mut_slice()
+    }
+
+    /// Return a mutable slice to the array. If the array was shared, this will make a copy of the array.
+    pub fn make_mut_slice(&mut self) -> &mut [T] {
         self.detach(self.len());
         unsafe { core::slice::from_raw_parts_mut(self.as_ptr() as *mut T, self.len()) }
     }
@@ -224,13 +234,7 @@ impl<T: Clone> SharedVector<T> {
         let mut inner = unsafe { self.inner.as_mut() };
 
         if inner.header.size >= new_len {
-            while inner.header.size > new_len {
-                inner.header.size -= 1;
-                // Safety: The array was of size inner.header.size, so there should be an element there
-                unsafe {
-                    core::ptr::drop_in_place(inner.data.as_mut_ptr().add(inner.header.size));
-                }
-            }
+            self.shrink(new_len);
         } else {
             while inner.header.size < new_len {
                 // Safety: The array must have a capacity of at least new_len because of the detach call earlier
@@ -241,6 +245,37 @@ impl<T: Clone> SharedVector<T> {
             }
         }
     }
+
+    fn shrink(&mut self, new_len: usize) {
+        if self.len() == new_len {
+            return;
+        }
+
+        assert!(
+            unsafe { self.inner.as_ref().header.refcount.load(atomic::Ordering::Relaxed) } == 1
+        );
+        // Safety: caller (and above debug_assert) must ensure that the array is not shared.
+        let mut inner = unsafe { self.inner.as_mut() };
+
+        while inner.header.size > new_len {
+            inner.header.size -= 1;
+            // Safety: The array was of size inner.header.size, so there should be an element there
+            unsafe {
+                core::ptr::drop_in_place(inner.data.as_mut_ptr().add(inner.header.size));
+            }
+        }
+    }
+
+    /// Clears the vector and removes all elements.
+    pub fn clear(&mut self) {
+        let is_shared =
+            unsafe { self.inner.as_ref().header.refcount.load(atomic::Ordering::Relaxed) } != 1;
+        if is_shared {
+            *self = SharedVector::default();
+        } else {
+            self.shrink(0)
+        }
+    }
 }
 
 impl<T> Deref for SharedVector<T> {
@@ -253,7 +288,7 @@ impl<T> Deref for SharedVector<T> {
 /* FIXME: is this a good idea to implement DerefMut knowing what it might detach?
 impl<T> DerefMut for SharedVector<T> {
     fn deref_mut(&mut self) -> &mut Self::Target {
-        self.as_slice_mut()
+        self.as_mut_slice()
     }
 }*/
 
@@ -310,7 +345,7 @@ impl<T> FromIterator<T> for SharedVector<T> {
                         while *begin < size {
                             unsafe {
                                 core::ptr::write(
-                                    result.inner.as_mut().data.as_mut_ptr().add(size),
+                                    result.inner.as_mut().data.as_mut_ptr().add(*begin),
                                     core::ptr::read(old_inner.as_ref().data.as_ptr().add(*begin)),
                                 );
                                 *begin += 1;
@@ -478,6 +513,76 @@ fn invalid_capacity_test() {
     let _: SharedVector<u8> = SharedVector::with_capacity(usize::MAX / 2 - 1000);
 }
 
+#[test]
+fn collect_from_iter_with_no_size_hint() {
+    struct NoSizeHintIter<'a> {
+        data: &'a [&'a str],
+        i: usize,
+    }
+
+    impl<'a> Iterator for NoSizeHintIter<'a> {
+        type Item = String;
+
+        fn next(&mut self) -> Option<Self::Item> {
+            if self.i >= self.data.len() {
+                return None;
+            }
+            let item = self.data[self.i];
+            self.i += 1;
+            Some(item.to_string())
+        }
+
+        fn size_hint(&self) -> (usize, Option<usize>) {
+            (0, None)
+        }
+    }
+
+    // 5 elements to be above the initial "grow"-capacity of 4 and thus require one realloc.
+    let input = NoSizeHintIter { data: &["Hello", "sweet", "world", "of", "iterators"], i: 0 };
+
+    let shared_vec: SharedVector<String> = input.collect();
+    assert_eq!(shared_vec.as_slice(), &["Hello", "sweet", "world", "of", "iterators"]);
+}
+
+#[test]
+fn test_capacity_grows_only_when_needed() {
+    let mut vec: SharedVector<u8> = SharedVector::with_capacity(2);
+    vec.push(0);
+    assert_eq!(vec.capacity(), 2);
+    vec.push(0);
+    assert_eq!(vec.capacity(), 2);
+    vec.push(0);
+    assert_eq!(vec.len(), 3);
+    assert!(vec.capacity() > 2);
+}
+
+#[test]
+fn test_vector_clear() {
+    let mut vec: SharedVector<String> = Default::default();
+    vec.clear();
+    vec.push("Hello".into());
+    vec.push("World".into());
+    vec.push("of".into());
+    vec.push("Vectors".into());
+
+    let mut copy = vec.clone();
+
+    assert_eq!(vec.len(), 4);
+    let orig_cap = vec.capacity();
+    assert!(orig_cap >= vec.len());
+    vec.clear();
+    assert_eq!(vec.len(), 0);
+    assert_eq!(vec.capacity(), 0); // vec was shared, so start with new empty vector.
+    vec.push("Welcome back".into());
+    assert_eq!(vec.len(), 1);
+    assert!(vec.capacity() >= vec.len());
+
+    assert_eq!(copy.len(), 4);
+    assert_eq!(copy.capacity(), orig_cap);
+    copy.clear(); // copy is not shared (anymore), retain capacity.
+    assert_eq!(copy.capacity(), orig_cap);
+}
+
 #[cfg(feature = "ffi")]
 pub(crate) mod ffi {
     use super::*;
