@@ -57,8 +57,7 @@ pub trait PlatformWindow {
     /// When set, `max_width` means that one need to wrap the text so it does not go further than that
     fn text_size(
         &self,
-        item_graphics_cache: &crate::item_rendering::CachedRenderingData,
-        unresolved_font_request_getter: &dyn Fn() -> crate::graphics::FontRequest,
+        font_request: crate::graphics::FontRequest,
         text: &str,
         max_width: Option<f32>,
     ) -> Size;
@@ -146,6 +145,7 @@ pub struct Window {
     cursor_blinker: RefCell<pin_weak::rc::PinWeak<crate::input::TextCursorBlinker>>,
 
     scale_factor: Pin<Box<Property<f32>>>,
+    active: Pin<Box<Property<bool>>>,
     active_popup: RefCell<Option<PopupWindow>>,
 }
 
@@ -172,6 +172,7 @@ impl Window {
             focus_item: Default::default(),
             cursor_blinker: Default::default(),
             scale_factor: Box::pin(Property::new(1.)),
+            active: Box::pin(Property::new(false)),
             active_popup: Default::default(),
         });
         let window_weak = Rc::downgrade(&window);
@@ -241,7 +242,7 @@ impl Window {
             .and_then(|(popup_component, coordinates)| {
                 event.translate(-coordinates.to_vector());
 
-                if let MouseEvent::MousePressed { pos } = &event {
+                if let MouseEvent::MousePressed { pos, .. } = &event {
                     // close the popup if one press outside the popup
                     let geom = ComponentRc::borrow_pin(&popup_component)
                         .as_ref()
@@ -339,6 +340,19 @@ impl Window {
         }
     }
 
+    /// Marks the window to be the active window. This typically coincides with the keyboard
+    /// focus. One exception though is when a popup is shown, in which case the window may
+    /// remain active but temporarily loose focus to the popup.
+    pub fn set_active(&self, active: bool) {
+        self.active.as_ref().set(active);
+    }
+
+    /// Returns true of the window is the active window. That typically implies having the
+    /// keyboard focus, except when a popup is shown and temporarily takes the focus.
+    pub fn active(&self) -> bool {
+        self.active.as_ref().get()
+    }
+
     /// If the component's root item is a Window element, then this function synchronizes its properties, such as the title
     /// for example, with the properties known to the windowing system.
     pub fn update_window_properties(&self) {
